# -*- coding: utf-8 -*-
"""
Views for newsletter subscription
"""
import json

from django.core.urlresolvers import reverse
from django.views.generic.edit import FormView
from django.views.generic.base import TemplateView
from django.http import HttpResponse

from newsletter_subscriptions.forms import SubscriptionForm
from newsletter_subscriptions.helpers import helper_finder, SubscriptionFormHelper


class AjaxableResponseMixin(object):
    """ Ajax form based on the django docs example.

    https://docs.djangoproject.com/en/dev/topics/class-based-views/generic-editing/#ajax-example
    https://docs.djangoproject.com/en/dev/topics/class-based-views/mixins/#more-than-just-html
    """

    def render_to_json_response(self, context, **response_kwargs):
        """Render a json response of the context."""

        data = json.dumps(context)
        response_kwargs['content_type'] = 'application/json'
        return HttpResponse(data, **response_kwargs)

    def form_invalid(self, form):
        response = super(AjaxableResponseMixin, self).form_invalid(form)
        if self.request.is_ajax():
            return self.render_to_json_response(form.errors, status=400)

        return response

    def form_valid(self, form):
        response = super(AjaxableResponseMixin, self).form_valid(form)
        if self.request.is_ajax():
            data = {
                'pk': self.subscribe_object.pk,
            }
            return self.render_to_json_response(data)

        return response


class SubscribeFormView(AjaxableResponseMixin, FormView):
    template_name = 'newsletter_subscriptions/form.html'
    success_urlname = 'newsletter-subscribe-sent'
    form_class = SubscriptionForm
    form_helper_class = SubscriptionFormHelper
    subscribe_object = None

    def get_form_helper_class(self):
        return helper_finder(self.form_helper_class)

    def get_form_helper(self):
        helper_class = self.get_form_helper_class()
        kwargs = self.get_form_helper_kwargs()
        return helper_class(**kwargs)

    def get_form_helper_kwargs(self):
        return {}

    def get_success_url(self):
        return reverse(self.success_urlname)

    def get_context_data(self, **kw):
        context = super(SubscribeFormView, self).get_context_data(**kw)
        context.update({
            'form_helper': self.get_form_helper(),
        })
        return context

    def form_valid(self, form):
        self.subscribe_object = form.save()
        return super(SubscribeFormView, self).form_valid(form)


class SubscribeConfirmView(TemplateView):
    template_name = 'newsletter_subscriptions/form_sent.html'
